@@ -1,20 +1,19 @@
 """
 Models and managers for tagging.
 """
-from django.db import models
-from django.db import connection
-from django.utils.encoding import smart_text
-from django.utils.encoding import python_2_unicode_compatible
-from django.utils.translation import ugettext_lazy as _
-from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes.fields import GenericForeignKey
+from django.contrib.contenttypes.models import ContentType
+from django.db import connection
+from django.db import models
+from django.utils.encoding import smart_text
+from django.utils.translation import gettext_lazy as _
 
 from tagging import settings
 from tagging.utils import LOGARITHMIC
-from tagging.utils import get_tag_list
 from tagging.utils import calculate_cloud
-from tagging.utils import parse_tag_input
 from tagging.utils import get_queryset_and_model
+from tagging.utils import get_tag_list
+from tagging.utils import parse_tag_input
 
 
 qn = connection.ops.quote_name
@@ -50,7 +49,11 @@ class TagManager(models.Manager):
         for tag_name in updated_tag_names:
             if tag_name not in current_tag_names:
                 tag, created = self.get_or_create(name=tag_name)
-                TaggedItem._default_manager.create(tag=tag, object=obj)
+                TaggedItem._default_manager.get_or_create(
+                    content_type_id=ctype.pk,
+                    object_id=obj.pk,
+                    tag=tag,
+                )
 
     def add_tag(self, obj, tag_name):
         """
@@ -431,12 +434,14 @@ class TaggedItemManager(models.Manager):
         GROUP BY %(model_pk)s
         ORDER BY %(count)s DESC
         %(limit_offset)s"""
+        tagging_table = qn(self.model._meta.get_field(
+            'tag').remote_field.model._meta.db_table)
         query = query % {
             'model_pk': '%s.%s' % (model_table, qn(model._meta.pk.column)),
             'count': qn('count'),
             'model': model_table,
             'tagged_item': qn(self.model._meta.db_table),
-            'tag': qn(self.model._meta.get_field('tag').rel.to._meta.db_table),
+            'tag': tagging_table,
             'content_type_id': content_type.pk,
             'related_content_type_id': related_content_type.pk,
             # Hardcoding this for now just to get tests working again - this
@@ -464,7 +469,6 @@ class TaggedItemManager(models.Manager):
 # Models #
 ##########
 
-@python_2_unicode_compatible
 class Tag(models.Model):
     """
     A tag.
@@ -484,7 +488,6 @@ class Tag(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class TaggedItem(models.Model):
     """
     Holds the relationship between a tag and the item being tagged.
@@ -492,11 +495,13 @@ class TaggedItem(models.Model):
     tag = models.ForeignKey(
         Tag,
         verbose_name=_('tag'),
-        related_name='items')
+        related_name='items',
+        on_delete=models.CASCADE)
 
     content_type = models.ForeignKey(
         ContentType,
-        verbose_name=_('content type'))
+        verbose_name=_('content type'),
+        on_delete=models.CASCADE)
 
     object_id = models.PositiveIntegerField(
         _('object id'),
